
#define AT91_PTRS	1
#include "../at91m40400.h"

#define TRUE	1
#define FALSE	0
#define RED_LED		(1<<1)
#define GREEN_LED	(1<<2)
#define AMBER_LED	(1<<4)

extern unsigned short prog_array[];
extern long prog_array_size();
/* Function prototypes */

void flash_ok();
void flash_not_ok();
void leds_on(void);
void leds_off(void);
void chip_erase();
long get_flash_id();
void sector_erase(short sector);
void block_erase(short block);
void program_word(long address,unsigned short word);
unsigned short read_word(long address);
void putlong(unsigned long val);
void putword(unsigned short val);
void puthex(unsigned char ch);
void putline(char *line);
void putc(char ch);

int main (void)
{
	int i;
	int ok;


	at91_pio->PIO_OER = 0x2 | 0x4 | 0x10;
	at91_pio->PIO_PDR = P14 | P15 | P21 | P22;
	leds_on();
	/* intialise the serial port*/
	at91_uart0->US_CR = RSTRX | RSTTX | RSTSTA;
	at91_uart0->US_CR = TXEN;
	at91_uart0->US_MR = USClk_MCK | EightBits | NoParity | Stop2Bit | NormalMode;
	at91_uart0->US_BRGR = 32; 	/* 38400 baud  20MHz/38400 * 16 */
	putline("\r\n\nAPS-01 Flash Programmer\r\n");
	get_flash_id();
	chip_erase();
	putline("Programming 0x");
	putlong(prog_array_size());
	putline(" bytes\r\n");
	for(i=0;i<prog_array_size();i++)
		program_word((long)i,prog_array[i]);
	putline("Programmed @ 0x");
	putlong(prog_array_size());
	putline("    \r\n");
	leds_off();
	ok = TRUE;
	for(i=0;i<prog_array_size();i++)
		if (prog_array[i] != read_word((long)i))
		{
			putline("Failed to program 0x");
			putword(prog_array[i]);
			putline(" Into addr 0x");
			putlong(i);
			putline("\r\n");
			ok = FALSE;
		}

	if (ok == TRUE)
	{
		putline("Flash programmed OK\r\n");
		flash_ok();
	}
	else
	{
		putline("Flash failed to program\r\n");
		flash_not_ok();
	}
	while(1==1);
	return(0);
}

void flash_ok()
{	
	int n,x;

	while(1)
	{
		leds_on();
		for(n=0;n<0xFFFF;n++)
			x++;
		for(n=0;n<0xFFFF;n++)
			x++;
		leds_off();
		for(n=0;n<0xFFFF;n++)
			x++;
	}
}

void flash_not_ok()
{	
	int n,x;

	while(1){
		leds_on();
		for(n=0;n<0xFFFF;n++)
			x++;
		leds_off();
		for(n=0;n<0xFFFF;n++)
			x++;
		for(n=0;n<0xFFFF;n++)
			x++;
	}
}

void leds_on(void)
{
	at91_pio->PIO_SODR = (RED_LED | GREEN_LED | AMBER_LED);
}

void leds_off(void)
{
	at91_pio->PIO_CODR = (RED_LED | GREEN_LED | AMBER_LED);
}

void __gccmain()
{
}

long get_flash_id()
{
	unsigned short *ptr,id,man;
	long size;
	
	ptr = (unsigned short *)FLASH_BASE;

	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0x90;

	id = ptr[1];
	man = ptr[0];

	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0xf0;
#ifdef SMALL_PROG
return(0);
#endif
	putline("Chip type ");
	switch (man &0xff)
	{
		case 0x01:	/* AMD */
			putline("AMD ");
			switch(id & 0xff)
			{
				case 0x20:	putline("AM29F010");
						size = 0x20000;
						break;
				case 0xa4:	putline("AM29F040");
						size = 0x80000;
						break;
				case 0x29:	putline("AM28F020A");
						size = 0x40000;
						break;
				case 0x2a:	putline("AM28F020");
						size = 0x40000;
						break;
				case 0xa2:	putline("AM28F010A");
						size = 0x20000;
						break;
				case 0xa7:	putline("AM28F010");
						size = 0x20000;
						break;
				case 0x25:	putline("AM28F512");
						size = 0x10000;
						break;
				case 0xa1:	putline("AM28F256");
						size = 0x8000;
						break;
				default:	putline("Unknown");
						size = 0;
						break;
			}
			break;
		case 0x1f:	/* ATMEL */
			putline("ATMEL ");
 			switch(id & 0xff)
			{
				case 0x35:	putline("AT29xx010A");
						size = 0x20000;
						break;
				case 0xba:	putline("AT29xx020");
						size = 0x40000;
						break;
				case 0xc4:	putline("AT29xx040A");
						size = 0x80000;
						break;
				case 0xbc:	putline("AT29LV256");
						size = 0x8000;
						break;
				case 0x3d:	putline("AT29LV512");
						size = 0x10000;
						break;
				case 0x26:	putline("AT29LV1024");
						size = 0x20000;
						break;
				case 0xdc:	putline("AT29C256");
						size = 0x8000;
						break;
				case 0x5d:	putline("AT29C512");
						size = 0x10000;
						break;
				case 0xd5:	putline("AT29C010A");
						size = 0x20000;
						break;
				case 0x25:	putline("AT29C1024");
						size = 0x20000;
						break;
				case 0xda:	putline("AT29C020");
						size = 0x40000;
						break;
				case 0xa4:	putline("AT29C040A");
						size = 0x80000;
						break;
				case 0x17:	putline("AT49F010");
						size = 0x20000;
						break;
				case 0x0b:	putline("AT49F020");
						size = 0x40000;
						break;
				case 0x82:	putline("AT49F2048");
						size = 0x40000;
						break;
				case 0x13:	putline("AT49F040");
						size = 0x80000;
						break;
				case 0x92:	putline("AT49F4096");
						size = 0x80000;
						break;
				case 0xa0:	putline("AT49F8192");
						size = 0x100000;
						break;
				default:	putline("Unknown");
						size = 0;
						break;
			}
			break;
		case 0x89:	/* INTEL */
			putline("INTEL ");
			putline("Unknown");
			size = 0;
			break;
		case 0xbf:	/* Silicon Storage Technology */
			putline("STS ");
 			switch(id & 0xff)
			{
				case 0x5d:	putline("SST 29EE512");
						size = 0x10000;
						break;
				case 0x3d:	putline("SST 29xE512");
						size = 0x10000;
						break;
				case 0x07:	putline("SST 29EE010");
						size = 0x20000;
						break;
				case 0x08:	putline("SST 29xE010");
						size = 0x20000;
						break;
				case 0x10:	putline("SST 29EE020");
						size = 0x40000;
						break;
				case 0x12:	putline("SST 29xE020");
						size = 0x40000;
						break;
				case 0x04:	putline("SST 28xF040");
						size = 0x80000;
						break;
				case 0x82:	putline("SST39VF160");
						size = 0x200000;
						break;
				default :	putline("Unknown 0x");
						puthex(id & 0xff);
						size = 0;
						break;
			}
			break;
		default:
			putline("Unknown");
			size = 0;
			break;
	}
	putline("\r\n");
	return(size);	
}

void chip_erase()
{
	unsigned short *ptr;
	int i;
	
	ptr = (unsigned short *)FLASH_BASE;

	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0x80;
	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0x10;
	i = 0;
	while(i != 2)
		if (ptr[0] != 0xffff)
			i = 0;
		else
			i++;
}

void sector_erase(short sector)
{
	unsigned short *ptr;
	
	ptr = (unsigned short *)FLASH_BASE;

	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0x80;
	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[sector] = 0x30;
}

void block_erase(short block)
{
	unsigned short *ptr;
	
	ptr = (unsigned short *)FLASH_BASE;

	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0x80;
	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[block] = 0x50;
}

void program_word(long address,unsigned short word)
{
	unsigned short *ptr;
	int i;

	if ((address & 0xf) == 0)
	{
		putline("Programming @ 0x");
		putlong(address);
		putline("\r");		
	}
	ptr = (unsigned short *)FLASH_BASE;

	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0xa0;
	ptr[address] = word;

	i = 0;
	while(i != 2)
		if (ptr[address] != word)
			i = 0;
		else
			i++;
	
}

unsigned short read_word(long address)
{
	unsigned short *ptr;
	
	ptr = (unsigned short *)FLASH_BASE;

	return(ptr[address]);
}

void putlong(unsigned long val)
{
	puthex((val >> 24) & 0xff);
	puthex((val >> 16) & 0xff);
	puthex((val >> 8) & 0xff);
	puthex(val & 0xff);
}

void putword(unsigned short val)
{
	puthex((val >> 8) & 0xff);
	puthex(val & 0xff);
}

void puthex(unsigned char ch)
{
	static char out[] ={'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};
	
	putc(out[ch >> 4]);
	putc(out[ch & 0x0f]);
}

void putline(char *line)
{
	while(*line != '\0')
		putc(*line++);
}

void putc(char ch)
{
	while ((at91_uart0->US_CSR & TXRDY) == 0);
	at91_uart0->US_THR = ch;
}

